#!/bin/bash
#
# Simple script to watch Java/Tomcat out of memory errors or dead Tomcats
# within multiple Tomcat installations. Run as a cron job, once per minute.
# Part of mimoza-tomcat5.5 package.

DEAD="engine is not running, but pid file exists"	# Error message to watch
DEFAULT="/etc/default"				# Defaults directory
ERROR="java.lang.OutOfMemoryError"		# Error message to watch
EXIT_ERR="1"					# Unsuccessfull exit signal
INIT="/etc/init.d"				# Init script directory
LASTLINES=4					# Tail size of logfiles scaned
LOGBASE="/var/log/tomcat5.5"			# Logs directory
LOGFILE1="localhost.`date +%Y-%m-%d`.log"	# Watched logfile #1
LOGFILE2="catalina.`date +%Y-%m-%d`.log"	# Watched logfile #2
LOGFILE3="restart.`date '+%Y-%m-%d'`.log"	# Own logfile
LOGGROUP="www-data"				# Own logfile Liux group
LOGOWNER="webadmin"				# Own logfile Linux user
LOGRIGHTS="640"					# Own logfile Linux rights
SLEEPTIME="5"					# Secs between restart and status
TOMCAT="tomcat5.5"				# Tomcat vesrion
TOMCAT_LIST=`ls -1 $DEFAULT/$TOMCAT.* 2>/dev/null | sed "s/^.*\/\(.*\.\)\(.*\)$/\2/"`
TOMCAT_SCRIPT="/usr/local/sbin/tomcat.sh"	# Multiple Tomcat handler script

# A beavatkozo scriptnek leteznie kell!

if [ ! -x $TOMCAT_SCRIPT ]; then
    echo -e "$0: Missing $TOMCAT_SCRIPT - exiting.\n"
    exit $EXIT_ERR
fi

# Sorbaveszi a letezo Tomcat-eket, es megvizsgalja:
# 1. a statust. Ha megtalalja a $DEAD szoveget, indit.
# 2. a logok utolso $LASTLINES sorat. Ha megtalalja az $ERROR szoveget, ujraindit.
# A beavatkozasokat logolja.

for current_tomcat in $TOMCAT_LIST
do
    logfile="$LOGBASE/$current_tomcat/$LOGFILE3"
    if [ ! -f  $logfile ]; then
	touch $logfile; chown $LOGOWNER:$LOGGROUP $logfile; chmod $LOGRIGHTS $logfile
    fi
    status=`$TOMCAT_SCRIPT $current_tomcat status`
    if [ ! -z "`$TOMCAT_SCRIPT $current_tomcat status | grep "$DEAD"`" ]; then
        echo "$0: Restarting dead $current_tomcat"
	echo "`date '+%Y.%m.%d %H:%M:%S'` Restarting dead $current_tomcat" >>$logfile
        $TOMCAT_SCRIPT $current_tomcat start
        sleep $SLEEPTIME
        $TOMCAT_SCRIPT $current_tomcat status
    else
        if [ -f $LOGBASE/$current_tomcat/$LOGFILE1 -o \
             -f $LOGBASE/$current_tomcat/$LOGFILE2 ]; then
            if [ ! -z "`tail -n $LASTLINES $LOGBASE/$current_tomcat/$LOGFILE1 2>/dev/null | grep -i $ERROR`" -o \
                 ! -z "`tail -n $LASTLINES $LOGBASE/$current_tomcat/$LOGFILE2 2>/dev/null | grep -i $ERROR`" ] ; then
                echo "$0: $ERROR detected - restarting $current_tomcat"
		echo "`date '+%Y.%m.%d %H:%M:%S'` $ERROR detected - restarting $current_tomcat" >>$logfile
                $TOMCAT_SCRIPT $current_tomcat restart
                sleep $SLEEPTIME
                $TOMCAT_SCRIPT $current_tomcat status
            fi
        fi
    fi
done
